#version 120
#include "lib/colorRange.glsl"

#define Bloom
	#define BloomMult 5.0 //[0.5 1.0 1.5 2.0 2.5 3.0 3.5 4.0 5.0 6.0 7.0 8.0] basic multiplier

//#define Vignette               //Dark tint around the screen
	#define VignetteMult 2.0 //[0.5 1.0 1.5 2.0 2.5 3.0 3.5 4.0 5.0 6.0 7.0 8.0] basic multiplier

#define ScreenRefraction               //Refracts the sceen underwater and when rain is hitting the camera.
	#define ScreenRefractionMult 1.0 //[0.5 1.0 1.5 2.0]

#define Brightness 1.00         //[0.0 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.55 0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95 1.00 1.05 1.10 1.15 1.20 1.25 1.30 1.35 1.40 1.45 1.50 1.55 1.60 1.65 1.70 1.75 1.80 1.85 1.90 1.95 2.00]
#define Gamma 1.00             //[0.0 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.55 0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95 1.00 1.05 1.10 1.15 1.20 1.25 1.30 1.35 1.40 1.45 1.50 1.55 1.60 1.65 1.70 1.75 1.80 1.85 1.90 1.95 2.00]
#define Contrast 1.00         //[0.0 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50 0.55 0.60 0.65 0.70 0.75 0.80 0.85 0.90 0.95 1.00 1.05 1.10 1.15 1.20 1.25 1.30 1.35 1.40 1.45 1.50 1.55 1.60 1.65 1.70 1.75 1.80 1.85 1.90 1.95 2.00]
#define Lift 0.0            //[0.05 0.1 0.15 0.2]
#define InverseLift 0.020 //[0.020 0.05 0.075 0.1]

#define RedMult 1.0 		  //[0.1 0.2 0.3 0.4 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define GreenMult 1.0 		 //[0.1 0.2 0.3 0.4 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define BlueMult 1.0 		//[0.1 0.2 0.3 0.4 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]

//#define DOF        //Intended for cinematic purposes
#define Focal 0.08 //[0.01 0.02 0.03 0.04 0.05 0.06 0.07 0.08 0.09 0.10 0.20 0.30 0.40 0.50 0.60 0.70 0.80 0.90 1.00 1.50 2.00 2.50 3.00 3.50 4.00 4.50 5.00 6.00 7.00 8.00 9.00 10.00 20.00 30.00 40.00 50.00 60.00 70.00 80.00 90.00 100.00]

#define FilmGrainMult 1.0 //[0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0 2.1 2.2 2.3 2.4 2.5 2.6 2.7 2.8 2.9 3.0 3.5 4.0 4.5 5.0]

const bool gcolorMipmapEnabled = true;

varying vec4 texcoord;

uniform sampler2D gcolor;
uniform sampler2D gdepthtex;
uniform sampler2D depthtex2;
uniform sampler2D composite;
uniform sampler2D noisetex;
uniform sampler2D colortex7;

uniform float frameTimeCounter;
uniform float viewWidth;
uniform float aspectRatio;
uniform float rainStrength;
uniform float centerDepthSmooth;

uniform ivec2 eyeBrightnessSmooth;

uniform float far;
uniform float near;

uniform int isEyeInWater;

float dynamicExposure = mix(1.0,0.0,(pow(eyeBrightnessSmooth.y / 240.0f, 3.0f)));

#include "lib/UniqueBiomes.glsl"

float getRainDrops(){

	float noiseTexture = texture2D(noisetex, vec2(texcoord.s, texcoord.t / 5.5) * 0.015 + vec2(0.0,frameTimeCounter) * 0.0005).x;
	noiseTexture += texture2D(noisetex, vec2(texcoord.s, texcoord.t / 5.5) * 0.03 + vec2(0.0,frameTimeCounter) * 0.002).x * 0.5;

	noiseTexture = min(max(noiseTexture - 1.1,0.0) * 10.0, 1.0);
	noiseTexture *= clamp((eyeBrightnessSmooth.y-220)/15.0,0.0,1.0);

	return noiseTexture;
}

vec2 customTexcoord(){
	vec2 texCoord = texcoord.st;

	float noiseTexture = texture2D(noisetex, texcoord.st * 0.03 + frameTimeCounter * 0.001).x;

	vec2 wavyness = (vec2(noiseTexture, noiseTexture) * 2.0 - 1.0) * mix(getRainDrops() * rainStrength * 2.0,1.0,isEyeInWater) * 0.0075 * BiomeRainDrops;

	#ifdef ScreenRefraction
		return texCoord + wavyness * ScreenRefractionMult;
	#else
		return texCoord;
	#endif
}

vec2 newTexcoord = customTexcoord();

vec3 tonemap(vec3 color)
{
	float a = Brightness;
	float b = Gamma;
	float c = Contrast;
	float d = Lift;
	float e = InverseLift;

	color = mix(color, vec3(0.33333), 1.2 - c) * (1.0 - rainStrength * 0.20);

	color = pow(color, vec3(2.2)) * pow(1.5, 2.2) * a * (1.0 - rainStrength * -0.80);

	vec3 x = max(vec3(0.0),color - InverseLift) + Lift;
	vec3 outCol = pow((x * (6.2 * x + 0.4)) / (x * (6.2 * x + 1.56) + 0.2), vec3(b));
	return outCol;
}

vec3 reinhardTonemap(vec3 color)
{
	return color / (1.0 + color);
}

float ld(float dist) {
    return (2.0 * near) / (far + near - dist * (far - near));
}

#ifdef Bloom

	vec3 getBloom(in vec2 bCoord){

		vec3 blur = vec3(0.0);

		float BloomPowMult = mix(1.0, 0.8, float(isEyeInWater));
		BloomPowMult = mix(BloomPowMult, 0.8, rainStrength * (1.0 - float(isEyeInWater)) * (1.0 - dynamicExposure));

		blur += pow(texture2D(composite,bCoord/pow(2.0,2.0) + vec2(0.0,0.0)).rgb,vec3(2.1) * BloomPowMult)*pow(8.0,1.0);
		blur += pow(texture2D(composite,bCoord/pow(2.0,3.0) + vec2(0.3,0.0)).rgb,vec3(2.1) * BloomPowMult)*pow(7.0,1.0);
		blur += pow(texture2D(composite,bCoord/pow(2.0,4.0) + vec2(0.0,0.3)).rgb,vec3(2.1) * BloomPowMult)*pow(6.0,1.0);
		blur += pow(texture2D(composite,bCoord/pow(2.0,5.0) + vec2(0.1,0.3)).rgb,vec3(2.1) * BloomPowMult)*pow(5.0,1.0);
		blur += pow(texture2D(composite,bCoord/pow(2.0,6.0) + vec2(0.2,0.3)).rgb,vec3(2.1) * BloomPowMult)*pow(4.0,1.0);

		return blur * 0.25;
	}

#endif

#ifdef Vignette
	vec3 getVignette(vec3 color, vec2 pos){
		float factor = distance(pos, vec2(0.5));

		factor = pow(factor, 4.4);
		factor *= VignetteMult;

		factor = clamp(1.0 - factor, 0.0, 1.0);

		return color * factor;
	}
#endif

#ifdef DOF
	//hexagon pattern
	const vec2 hex_offsets[60] = vec2[60]  (  vec2( 0.0000, 0.2500 ),
								vec2( -0.2165, 0.1250 ),
								vec2( -0.2165, -0.1250 ),
								vec2( -0.0000, -0.2500 ),
								vec2( 0.2165, -0.1250 ),
								vec2( 0.2165, 0.1250 ),
								vec2( 0.0000, 0.5000 ),
								vec2( -0.2500, 0.4330 ),
								vec2( -0.4330, 0.2500 ),
								vec2( -0.5000, 0.0000 ),
								vec2( -0.4330, -0.2500 ),
								vec2( -0.2500, -0.4330 ),
								vec2( -0.0000, -0.5000 ),
								vec2( 0.2500, -0.4330 ),
								vec2( 0.4330, -0.2500 ),
								vec2( 0.5000, -0.0000 ),
								vec2( 0.4330, 0.2500 ),
								vec2( 0.2500, 0.4330 ),
								vec2( 0.0000, 0.7500 ),
								vec2( -0.2565, 0.7048 ),
								vec2( -0.4821, 0.5745 ),
								vec2( -0.6495, 0.3750 ),
								vec2( -0.7386, 0.1302 ),
								vec2( -0.7386, -0.1302 ),
								vec2( -0.6495, -0.3750 ),
								vec2( -0.4821, -0.5745 ),
								vec2( -0.2565, -0.7048 ),
								vec2( -0.0000, -0.7500 ),
								vec2( 0.2565, -0.7048 ),
								vec2( 0.4821, -0.5745 ),
								vec2( 0.6495, -0.3750 ),
								vec2( 0.7386, -0.1302 ),
								vec2( 0.7386, 0.1302 ),
								vec2( 0.6495, 0.3750 ),
								vec2( 0.4821, 0.5745 ),
								vec2( 0.2565, 0.7048 ),
								vec2( 0.0000, 1.0000 ),
								vec2( -0.2588, 0.9659 ),
								vec2( -0.5000, 0.8660 ),
								vec2( -0.7071, 0.7071 ),
								vec2( -0.8660, 0.5000 ),
								vec2( -0.9659, 0.2588 ),
								vec2( -1.0000, 0.0000 ),
								vec2( -0.9659, -0.2588 ),
								vec2( -0.8660, -0.5000 ),
								vec2( -0.7071, -0.7071 ),
								vec2( -0.5000, -0.8660 ),
								vec2( -0.2588, -0.9659 ),
								vec2( -0.0000, -1.0000 ),
								vec2( 0.2588, -0.9659 ),
								vec2( 0.5000, -0.8660 ),
								vec2( 0.7071, -0.7071 ),
								vec2( 0.8660, -0.5000 ),
								vec2( 0.9659, -0.2588 ),
								vec2( 1.0000, -0.0000 ),
								vec2( 0.9659, 0.2588 ),
								vec2( 0.8660, 0.5000 ),
								vec2( 0.7071, 0.7071 ),
								vec2( 0.5000, 0.8660 ),
								vec2( 0.2588, 0.9659 ));

	vec3 getDof(inout vec3 color){

	const float focal = Focal;
	float aperture = (35.0 / 6.0) / 1000.0;
	const float sizemult = 50.0;

		float DoFGamma = 4.545;
				//Calculate pixel Circle of Confusion that will be used for bokeh depth of field
				float z = texture2D(depthtex2, texcoord.st).x;
				float focus = centerDepthSmooth;
				float pcoc = (z-focus)/15 * focal * 10;
				vec4 sample = vec4(0.0);
				vec3 bcolor = vec3(0.0);
				float nb = 0.0;
				vec2 bcoord = vec2(0.0);

				for ( int i = 0; i < 60; i++) {
					sample = texture2D(gcolor, newTexcoord.xy + hex_offsets[i]*pcoc*vec2(1.0,aspectRatio),abs(pcoc * 125.0));

					sample.rgb *= MaxColorRange;

					bcolor += pow(sample.rgb, vec3(DoFGamma));
				}
		color.rgb = pow(bcolor/60.0, vec3(1.0/DoFGamma));

	return color;
	}
#endif

void main(){

	vec3 color = texture2D(gcolor, newTexcoord.st).rgb * MaxColorRange;

	#ifdef DOF
		 color = getDof(color);
	#endif

	color += (texture2D(noisetex,texcoord.xy*vec2(aspectRatio,1.0)+4.0*frameTimeCounter).rgb*2.0-1.0)/128.0*FilmGrainMult;

	#ifdef Bloom
		color.rgb = pow(color.rgb, vec3(2.2));
		color += pow(reinhardTonemap(getBloom(newTexcoord.st)) * MaxColorRange * 0.02 * BloomMult / reinhardTonemap(vec3(1.0)), vec3(2.2)) * 3.0;
		color.rgb = pow(color.rgb, vec3(0.4545));
	#endif

	color.r *= RedMult;
	color.g *= GreenMult;
	color.b *= BlueMult;

	color.rgb = tonemap(color.rgb);

	#ifdef Vignette
		color = getVignette(color, newTexcoord);
	#endif

/*DRAWBUFFERS:1*/
	gl_FragData[0] = vec4(color, 1.0);
}
