#version 120
vec4 waterColor = vec4(1.0,1.0,1.0,0.24);

#define ParallaxWater //Makes waves on water look 3D
	#define WaterDepth 1.5 //[0.5 1.0 1.5 2.0 2.5 3.0]
	#define WaterPoints 4 //[2 4 6 8 16 32] //Slight quality increase, insignificant

varying vec4 texcoord;
varying vec4 lmcoord;
varying vec4 color;

varying vec3 normal;

varying vec3 viewVector;
varying vec3 wpos;
varying mat3 tbnMatrix;

varying float dist;
varying float mat;

uniform sampler2D texture;

uniform float frameTimeCounter;
uniform float rainStrength;

uniform int worldTime;

float timefract = worldTime;

mat2 time = mat2(vec2(
				((clamp(timefract, 23000.0f, 25000.0f) - 23000.0f) / 1000.0f) + (1.0f - (clamp(timefract, 0.0f, 2000.0f)/2000.0f)),
				((clamp(timefract, 0.0f, 2000.0f)) / 2000.0f) - ((clamp(timefract, 9000.0f, 12000.0f) - 9000.0f) / 3000.0f)),

				vec2(

				((clamp(timefract, 9000.0f, 12000.0f) - 9000.0f) / 3000.0f) - ((clamp(timefract, 12000.0f, 12750.0f) - 12000.0f) / 750.0f),
				((clamp(timefract, 12000.0f, 12750.0f) - 12000.0f) / 750.0f) - ((clamp(timefract, 23000.0f, 24000.0f) - 23000.0f) / 1000.0f))
);	//time[0].xy = sunrise and noon. time[1].xy = sunset and mindight.

float transition_fading = 1.0-(clamp((timefract-12000.0)/300.0,0.0,1.0)-clamp((timefract-13000.0)/300.0,0.0,1.0) + clamp((timefract-22000.0)/200.0,0.0,1.0)-clamp((timefract-23400.0)/200.0,0.0,1.0));


#include "lib/UniqueBiomes.glsl"
#include "lib/waterBump.glsl"

#ifdef ParallaxWater
vec3 getParallaxDisplacement(vec3 posxz, float iswater) {

	float waveZ = mix(2.0,0.25,iswater);
	float waveM = 2.0 * iswater;

	vec2 offset = viewVector.xy * (12.5 * WaterDepth) / dist / float(WaterPoints);
	
	for(int i = 0; i < WaterPoints; i++){
		posxz.xz = getWaterBump(posxz.xz - posxz.y, waveM, waveZ , iswater) * offset + posxz.xz;
	}
	return posxz;
}
#endif

void main(){
	float iswater = float(mat > 0.1 && mat < 0.29);
	
	vec3 posxz = wpos.xyz;

	#ifdef ParallaxWater	
		posxz = getParallaxDisplacement(posxz, iswater);
	#endif

	vec4 albedo = texture2D(texture, texcoord.st);
		 albedo = mix(albedo * color, waterColor, iswater);

	vec3 bump;
		bump = getWaveHeight(posxz.xz - posxz.y,iswater);
	
	const float bumpmult = 0.2;
	
	bump = bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
						  
	vec4 normalTangentSpace = vec4(normalize(bump * tbnMatrix) * 0.5 + 0.5, 1.0);
	
	#include "lib/lmCoord.glsl"
	
/* DRAWBUFFERS:531 */

	gl_FragData[0] = albedo;
	gl_FragData[1] = normalTangentSpace;
	gl_FragData[2] = vec4(lightmaps.x, mat, lightmaps.y, 1.0);
	
}