#version 120
#extension GL_ARB_shader_texture_lod : enable

#include "lib/options.glsl"

const vec3 intervalMult = vec3(1.0, 1.0, 1.0/(POMDepth / 8.0))/POMMapRes * 64 / POMQuality;
const float MAX_OCCLUSION_DISTANCE = 64.0;
const float MIX_OCCLUSION_DISTANCE = 12.0;
const int   MAX_OCCLUSION_POINTS   = POMQuality;

varying vec4 texcoord;
varying vec4 lmcoord;
varying vec4 color;
varying vec4 vtexcoordam;
varying vec4 vtexcoord;

varying vec3 normal;
varying vec3 tangent;
varying vec3 binormal;
varying vec3 viewVector;

varying vec3 wpos;

varying float mat;
varying float dist;

uniform sampler2D texture;
uniform sampler2D normals;
uniform sampler2D specular;
uniform sampler2D noisetex;

uniform vec3 upPosition;

uniform float wetness;
uniform float frameTimeCounter;

#include "lib/terrainBump.glsl"

#ifdef RainPuddles
	#include "lib/rainPuddles.glsl"
#endif

const float mincoord = 1.0/4096.0;
const float maxcoord = 1.0-mincoord;

vec2 dcdx = dFdx(vtexcoord.st*vtexcoordam.pq);
vec2 dcdy = dFdy(vtexcoord.st*vtexcoordam.pq);

vec4 readTexture(vec2 coord)
{
	return texture2DGradARB(texture,fract(coord)*vtexcoordam.pq+vtexcoordam.st,dcdx,dcdy);
}

vec4 readNormal(vec2 coord)
{
	return texture2DGradARB(normals,fract(coord)*vtexcoordam.pq+vtexcoordam.st,dcdx,dcdy);
}

void main(){

	vec2 adjustedTexCoord = texcoord.st;
	vec2 newcoord = vtexcoord.st*vtexcoordam.pq+vtexcoordam.st;
	vec2 coord = vtexcoord.st;
	
	float pomfade = clamp((dist-MAX_OCCLUSION_DISTANCE)/32.0,0.0,1.0);

	vec4 albedo = texture2D(texture, newcoord.st) * color * TextureBrightness * 0.8;
	vec3 bump = (texture2D(normals, newcoord.st).rgb * 2.0 - 1.0);

	#ifdef POM
	if (dist < MAX_OCCLUSION_DISTANCE+32.0) {
		if (viewVector.z < 0.0 && readNormal(vtexcoord.st).a < (1.0-1.0/POMQuality) && readNormal(vtexcoord.st).a > 1.0/POMQuality){
			vec2 interval = viewVector.xy * 0.05 * (1.0-pomfade) * POMDepth / (-viewVector.z * POMQuality);
			for (int i = 0; i < POMQuality; i++) {
				if (readNormal(coord).a < 1.0-float(i)/POMQuality) coord = coord+interval;
				else break;
			}
			if (coord.t < mincoord) {
				if (readTexture(vec2(coord.s,mincoord)).a == 0.0) {
					coord.t = mincoord;
					discard;
			}
		}
			newcoord = fract(coord.st)*vtexcoordam.pq+vtexcoordam.st;
			albedo = texture2DGradARB(texture, newcoord,dcdx,dcdy) * color;
		}
	}
	#endif


	#include "lib/lmCoord.glsl"

	vec2 posxz = wpos.xz - wpos.y;
	vec3 newnormal = normal;
	vec3 normalmap = texture2DGradARB(normals,newcoord.xy,dcdx,dcdy).xyz*2.0-1.0;
	mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
							tangent.y, binormal.y, normal.y,
							tangent.z, binormal.z, normal.z);
	
	newnormal = normalize(normalmap * tbnMatrix);

	#ifdef RainPuddles
		float rainpuddles = getRainPuddles(wpos) * smoothstep(0.8,0.9,lightmaps.y) * clamp(dot(normalize(upPosition), normal),0.0,1.0);
	#else
		float rainpuddles = 1.0;
	#endif
	
	#ifdef SpecularMapping
		vec3 specularity = texture2D(specular, newcoord).rgb;
	#else 
		vec3 specularity = vec3(0.0);
	#endif
	
	float bumpmult = 0.4 * (1.0 - clamp(rainpuddles * 1.5, 0.0, 1.0));

	bump = bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
	
	bump = getTerrainHeight(posxz) * (0.05 * rainpuddles) + bump;
						  
	vec4 normalTangentSpace = vec4(normalize(bump * tbnMatrix) * 0.5 + 0.5, 1.0);

/* DRAWBUFFERS:0246 */

	gl_FragData[0] = albedo;
	gl_FragData[1] = normalTangentSpace;
	gl_FragData[2] = vec4(lightmaps.x, mat, lightmaps.y, 1.0);
	#ifdef SpecularMapping
		gl_FragData[3] = vec4(specularity, 1.0);
	#endif
	
}